#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script for reading in polynomial data from the csv files.

@author: mark
"""

import logging
import numpy as np
import csv

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

def parse_polynomial_data_file_list(filename_list):
    """This parses each file, and then returns the dictionary containing the
    information, by solvent ID.
    """
    polynomial_data_by_solventid ={}
    for filename in filename_list:
        solvent_id = generate_solvent_id(filename)
        polynomial_fit_data = parse_polynomial_data_file(filename)
        polynomial_data_by_solventid[solvent_id] = polynomial_fit_data
    return polynomial_data_by_solventid

def generate_solvent_id(filename):
    """This gets the solvent ID from the filename. assumes file name is of the
    form '*/solventID.csv' where 'solventID' is the solvent ID.
    """
    return filename.split('/')[-1].replace('.csv', '')

def parse_polynomial_data_file(filename):
    """This reads in the file and parses the polynomial lines.
    """
    polynomial_data_dict = {}
    data_in = read_polynomial_data_file(filename)
    for line_list in data_in[1:]:
        polynomial_data = read_polynomial_data_line(line_list)
        polynomial_data_dict[polynomial_data["order"]] = polynomial_data
    return polynomial_data_dict

def read_polynomial_data_file(filename):
    """This reads in the tab separated variable file containing the polynomial fit data.
    """
    data_in = []
    with open(filename, "r") as file_in:
        file_contents = csv.reader(file_in, delimiter='\t')
        for line in file_contents:
            data_in.append(line)
    return data_in

def read_polynomial_data_line(line_list):
    """This reads in the values to a dictionary.
    This assumes that the values in the line are formatted such that they are
    ordered:
        Order, RMSE, covariance, coefficients.
    """
    order = int(line_list[0])
    RMSE = float(line_list[1])
    covariance = float(line_list[2])
    coefficients = np.array([float(x) for x in line_list[3:]])
    return {"order": order, "RMSE":RMSE, "covar":covariance,
            "coefficients":coefficients}
