#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script for the comparison of the polynomial fits of different solvents.

@author: mark
"""

import logging
import numpy as np
import numpy.polynomial.polynomial as poly
import scipy.spatial.distance as sciptdist

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

def generate_similarity_matrix_plot_input(sorted_id_list, matrix):
    """This compiles the input information needed for the similarity matrix plot.
    """
    return {"labels":sorted_id_list, "value_matrix":matrix}

def calculate_hausdorff_distance_matrix_from_poly_coeff(x_values, polynomial_order, poly_data_by_id):
    """This first extracts and calculates the values, then returns the Hausdorff matrix.
    """
    polynomial_values_by_id = calculate_polynomial_values_all_solvents(x_values, polynomial_order, poly_data_by_id)
    return calculate_hausdorff_distance_matrix(x_values, polynomial_values_by_id)

def calculate_hausdorff_distance_matrix(x_values, polynomial_values_dict_by_solvent_id):
    """This calculates the hausdorff distance between all polynomial curves.
    """
    sorted_id_list = get_sorted_solvent_id_list(polynomial_values_dict_by_solvent_id)
    hausdorff_matrix = np.zeros((len(sorted_id_list), len(sorted_id_list)))
    for i in range(len(sorted_id_list)):
        poly_values_i = polynomial_values_dict_by_solvent_id[sorted_id_list[i]]
        for j in range(i):
            poly_values_j = polynomial_values_dict_by_solvent_id[sorted_id_list[j]]
            hausdorff_value = calculate_hausdorff_distance(x_values, poly_values_i, poly_values_j)
            if i == j:
                hausdorff_matrix[i][i] = hausdorff_value
            else:
                hausdorff_matrix[i][j] = hausdorff_value
                hausdorff_matrix[j][i] = hausdorff_value
    return hausdorff_matrix

def calculate_rmsd_matrix_from_poly_coeff(x_values, polynomial_order, poly_data_by_id):
    """This first extracts and calculates the values, then returns the RMSD matrix.
    """
    polynomial_values_by_id = calculate_polynomial_values_all_solvents(x_values, polynomial_order, poly_data_by_id)
    return calculate_rmsd_matrix(polynomial_values_by_id)

def calculate_rmsd_matrix(polynomial_values_dict_by_solvent_id):
    """This calculates the rmsd between all curves, and outputs a matrix.
    """
    sorted_id_list = get_sorted_solvent_id_list(polynomial_values_dict_by_solvent_id)
    rmsd_matrix = np.zeros((len(sorted_id_list), len(sorted_id_list)))
    for i in range(len(sorted_id_list)):
        poly_values_i = polynomial_values_dict_by_solvent_id[sorted_id_list[i]]
        for j in range(i):
            poly_values_j = polynomial_values_dict_by_solvent_id[sorted_id_list[j]]
            rmsd_value = calculate_rmsd_between_curves(poly_values_i, poly_values_j)
            #Noting that the matrix is symmetric
            if i == j:
                rmsd_matrix[i][i] = rmsd_value
            else:
                rmsd_matrix[i][j] = rmsd_value
                rmsd_matrix[j][i] = rmsd_value
    return rmsd_matrix

def get_sorted_solvent_id_list(polynomial_dict_by_solvent_id):
    """This returns a sorted list ofthe solvent IDs.
    """
    return sorted(polynomial_dict_by_solvent_id.keys())

def calculate_hausdorff_distance(x_values, poly_values_1, poly_values_2):
    """This calculates the Hausdorff distance between the 2 polynomial curves.
    """
    poly_values_1_with_x = create_array_for_hausdorff_analysis(x_values, poly_values_1)
    poly_values_2_with_x = create_array_for_hausdorff_analysis(x_values, poly_values_2)
    return max(sciptdist.directed_hausdorff(poly_values_1_with_x, poly_values_2_with_x)[0],
               sciptdist.directed_hausdorff(poly_values_2_with_x, poly_values_1_with_x)[0])

def calculate_rmsd_between_curves(poly_values_1, poly_values_2):
    """This calculates the RMSD between the Assoicaiton energies along
    the both curves. Note that both arrays must have the same dimensionality
    for this analysis.
    """
    return np.sqrt(((poly_values_1 - poly_values_2)**2).mean())

def create_array_for_hausdorff_analysis(x_values, polynomial_values):
    """This creates a array of the correct dimensionality for hausdorff analysis.
    """
    array_size_2n = np.array([x_values, polynomial_values])
    array_size_n2 = array_size_2n.transpose()
    return array_size_n2

def calculate_polynomial_values_all_solvents(x_values, polynomial_order,
                                             polynomial_data_by_solvent_id):
    """This calculates the polynomial vlaues for each solvent ID, and creates
    a new dictionary, where the key is the solvent ID, and te value is the
    polynomial values at the given x points.
    """
    poly_values_by_solvent_id = {}
    for solvent_id, solvent_info_dict in polynomial_data_by_solvent_id.items():
        poly_values = calculate_polynomial_values_from_dict(x_values, solvent_info_dict, polynomial_order)
        poly_values_by_solvent_id[solvent_id] = poly_values
    return poly_values_by_solvent_id

def calculate_polynomial_values_from_dict(x_values, solvent_info_dict, polynomial_order):
    """This calculates the polynomial values for the given info dict.
    """
    polynomial_coefficients = get_polynomial_coefficients(solvent_info_dict, polynomial_order)
    return calculate_polynomial_values(x_values, polynomial_coefficients)

def get_polynomial_coefficients(solvent_info_dict, polynomial_order):
    """This returns the polynomial coefficients for the selected order.
    """
    return solvent_info_dict[polynomial_order]["coefficients"]


def calculate_polynomial_values(x_values, polynomial_coefficients):
    """This returns the values for the given polynomial function coefficients.
    """
    return poly.polyval(x_values, polynomial_coefficients)
