#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script for analysing the polynomial fitting data.

This will provide tools to analyse which order gives the best fit to the data.

@author: mark
"""

import logging
import numpy as np

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

def get_total_sorted_by_alt_metric(full_order_list, best_order_sorted_dict):
    """This gets the sorted information and orders them ready to plot them.
    Need to feed in a list of all the orders, as not all may appear.
    """
    numbers_by_order = []
    for i in range(len(full_order_list)):
        order = full_order_list[i]
        order_info = []
        for j in range(len(full_order_list)):
            
            sorted_dict_order = full_order_list[j]
            if sorted_dict_order in best_order_sorted_dict.keys():
                if order in best_order_sorted_dict[sorted_dict_order].keys():
                    order_info.append(len(best_order_sorted_dict[sorted_dict_order][order]))
                else:
                    order_info.append(0)
            else:
                order_info.append(0)
        numbers_by_order.append(tuple(order_info))
    numbers_by_order_tuple = tuple(numbers_by_order)
    return (tuple(full_order_list), numbers_by_order_tuple)

def get_total_by_fit_order(best_order_dict):
    """This returns a tuple containing the order, and also the number of
    occurances where the corresponding order provided the best fit.
    """
    order_tuple = tuple(sorted(best_order_dict.keys()))
    number_by_order = tuple([len(best_order_dict[order]) for order in order_tuple])
    return (order_tuple, number_by_order)

def generate_lists_by_alt_metric(solvent_ids_by_order, solvent_ids_by_order_alt_metric):
    """This goes through each order in the data and uses the best polynomial
    fit data to sort the solvent_ids by the alternative metric.
    """
    solvent_ids_sorted_by_alt_metric = {}
    for order, solvent_id_list in solvent_ids_by_order.items():
        solvent_ids_sorted_by_alt_metric[order] = extract_lists_by_order_of_alt_metric(solvent_id_list, solvent_ids_by_order_alt_metric)
    return solvent_ids_sorted_by_alt_metric

def extract_lists_by_order_of_alt_metric(solvent_id_list, solvent_ids_by_order):
    """This returns a dictionary where the solvent IDs are sorted based on the
    best order from the alternate metric.
    """
    solvent_id_alt_metric = {}
    for order in solvent_ids_by_order.keys():
        metric_ids_for_order = []
        for solvent_id in solvent_id_list:
            if solvent_id in solvent_ids_by_order[order]:
                metric_ids_for_order.append(solvent_id)
        solvent_id_alt_metric[order] = metric_ids_for_order
    return solvent_id_alt_metric

def get_solvent_id_by_order(best_order_dict):
    """This gets the values from the best order dict and returns lists of
    solvent IDs group by the order of the polynomial that gives the best fit.
    """
    solvent_ids_by_order = {}
    for order in best_order_dict.keys():
        solvent_ids_by_order[order] = get_solvent_id(best_order_dict[order])
    return solvent_ids_by_order

def get_solvent_id(order_info_list):
    """This gets the values from the order info list about which solvents are in this set.
    """
    return [order_info_list[i][0] for i in range(len(order_info_list))]

def get_best_poly_fit_rmse(polynomial_dict_by_solvent_id):
    """This returns a dict with tuples containing the information.
    """
    best_rmse_order_dict = {}
    for solvent_id, polynomial_dict in polynomial_dict_by_solvent_id.items():
        best_order, smallest_rmse = extract_best_poly_fit_rmse(polynomial_dict)
        info_tuple = (solvent_id, smallest_rmse)
        if best_order not in best_rmse_order_dict.keys():
            best_rmse_order_dict[best_order] = [info_tuple]
        else:
            best_rmse_order_dict[best_order].append(info_tuple)
    return best_rmse_order_dict

def extract_best_poly_fit_rmse(polynomial_dict):
    """This returns the order with the lowest value of RMSE,
    and value of the RMSE.
    """
    best_order = None
    smallest_rmse = None
    for order, info in polynomial_dict.items():
        if smallest_rmse == None:
            smallest_rmse = info["RMSE"]
            best_order = order
        elif smallest_rmse > info["RMSE"]:
            smallest_rmse = info["RMSE"]
            best_order = order
    return best_order, smallest_rmse

def get_best_poly_fit_covar(polynomial_dict_by_solvent_id):
    """This returns a dict with tuples containing the information.
    """
    best_covar_order_dict = {}
    for solvent_id, polynomial_dict in polynomial_dict_by_solvent_id.items():
        best_order, smallest_covar = extract_best_poly_fit_covar(polynomial_dict)
        info_tuple = (solvent_id, smallest_covar)
        if best_order not in best_covar_order_dict.keys():
            best_covar_order_dict[best_order] = [info_tuple]
        else:
            best_covar_order_dict[best_order].append(info_tuple)
    return best_covar_order_dict

def extract_best_poly_fit_covar(polynomial_dict):
    """This returns the order with the lowest covariance, and the
    value of covariance.
    """
    best_order = None
    smallest_covar = None
    for order, info in polynomial_dict.items():
        if smallest_covar == None:
            smallest_covar = info["covar"]
            best_order = order
        elif smallest_covar > info["covar"]:
            smallest_covar = info["covar"]
            best_order = order
    return best_order, smallest_covar